# Copyright 2013 VMware, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import contextlib
import mock
import webob.exc

from neutron.common import config as n_config
from neutron import context
from neutron.db.firewall import firewall_db
from neutron.openstack.common import uuidutils
from neutron.plugins.vmware.vshield.common import exceptions as vcns_exc
from neutron.plugins.vmware.vshield import vcns_driver
from neutron.tests.unit.db.firewall import test_db_firewall
from neutron.tests.unit.vmware import get_fake_conf
from neutron.tests.unit.vmware import VCNS_NAME
from neutron.tests.unit.vmware.vshield import fake_vcns


_uuid = uuidutils.generate_uuid

VSE_ID = 'edge-1'
ROUTER_ID = '42f95450-5cc9-44e4-a744-1320e592a9d5'

VCNS_CONFIG_FILE = get_fake_conf("vcns.ini.test")


class VcnsDriverTestCase(test_db_firewall.FirewallPluginDbTestCase,
                         firewall_db.Firewall_db_mixin):

    def vcns_firewall_patch(self):
        instance = self.mock_vcns.start()
        instance.return_value.update_firewall.side_effect = (
            self.fc2.update_firewall)
        instance.return_value.delete_firewall.side_effect = (
            self.fc2.delete_firewall)
        instance.return_value.update_firewall_rule.side_effect = (
            self.fc2.update_firewall_rule)
        instance.return_value.delete_firewall_rule.side_effect = (
            self.fc2.delete_firewall_rule)
        instance.return_value.add_firewall_rule_above.side_effect = (
            self.fc2.add_firewall_rule_above)
        instance.return_value.add_firewall_rule.side_effect = (
            self.fc2.add_firewall_rule)
        instance.return_value.get_firewall.side_effect = (
            self.fc2.get_firewall)
        instance.return_value.get_firewall_rule.side_effect = (
            self.fc2.get_firewall_rule)

    def setUp(self):

        n_config.parse(['--config-file', VCNS_CONFIG_FILE])
        # mock vcns
        self.fc2 = fake_vcns.FakeVcns(unique_router_name=False)
        self.mock_vcns = mock.patch(VCNS_NAME, autospec=True)
        self.vcns_firewall_patch()

        self.driver = vcns_driver.VcnsDriver(mock.Mock())

        super(VcnsDriverTestCase, self).setUp()
        self.addCleanup(self.fc2.reset_all)
        self.addCleanup(self.mock_vcns.stop)

        self.tenant_id = _uuid()
        self.subnet_id = _uuid()


class TestEdgeFwDriver(VcnsDriverTestCase):

    def _make_firewall_dict_with_rules(self, context, firewall_id):
        fw = self.get_firewall(context, firewall_id)
        fw_policy_id = fw['firewall_policy_id']
        if fw_policy_id:
            firewall_policy_db = self._get_firewall_policy(
                context, fw_policy_id)
            fw['firewall_rule_list'] = [
                self._make_firewall_rule_dict(fw_rule_db)
                for fw_rule_db in firewall_policy_db['firewall_rules']
            ]

        return fw

    def _compare_firewall_rule_lists(self, firewall_policy_id,
                                     list1, list2):
        for r1, r2 in zip(list1, list2):
            rule = r1['firewall_rule']
            rule['firewall_policy_id'] = firewall_policy_id
            for k in rule:
                self.assertEqual(rule[k], r2[k])

    def test_create_and_get_firewall(self):
        ctx = context.get_admin_context()
        name = 'firewall'
        with contextlib.nested(self.firewall_rule(name='fwr1',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr2',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr3',
                                                  no_delete=True)) as fr:
            fw_rule_ids = [r['firewall_rule']['id'] for r in fr]
            with self.firewall_policy(firewall_rules=fw_rule_ids,
                                      no_delete=True) as fwp:
                fwp_id = fwp['firewall_policy']['id']
                with self.firewall(name=name,
                                   firewall_policy_id=fwp_id) as firewall:
                    fw_create = firewall['firewall']
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_expect)
                    fw_get = self.driver.get_firewall(ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])

    def test_update_firewall_with_rules(self):
        ctx = context.get_admin_context()
        name = 'new_firewall'
        with contextlib.nested(self.firewall_rule(name='fwr1',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr2',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr3',
                                                  no_delete=True)) as fr:
            fw_rule_ids = [r['firewall_rule']['id'] for r in fr]
            with self.firewall_policy(firewall_rules=fw_rule_ids,
                                      no_delete=True) as fwp:
                fwp_id = fwp['firewall_policy']['id']
                with self.firewall(name=name,
                                   firewall_policy_id=fwp_id) as firewall:
                    fw_create = firewall['firewall']
                    fw_create = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_create)

                    data = {'firewall_rule': {'name': name,
                                              'source_port': '10:20',
                                              'destination_port': '30:40'}}
                    self.new_update_request('firewall_rules', data,
                                            fr[0]['firewall_rule']['id'])
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_expect)

                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])

    def test_delete_firewall(self):
        ctx = context.get_admin_context()
        name = 'firewall'
        with contextlib.nested(self.firewall_rule(name='fwr1',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr2',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr3',
                                                  no_delete=True)) as fr:
            fw_rule_ids = [r['firewall_rule']['id'] for r in fr]
            with self.firewall_policy(firewall_rules=fw_rule_ids,
                                      no_delete=True) as fwp:
                fwp_id = fwp['firewall_policy']['id']
                with self.firewall(name=name,
                                   firewall_policy_id=fwp_id) as firewall:
                    fw_create = firewall['firewall']
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_expect)
                    self.driver.delete_firewall(ctx, VSE_ID)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self.assertFalse(fw_get['firewall_rule_list'])

    def test_update_firewall_rule(self):
        ctx = context.get_admin_context()
        name = 'new_firewall'
        with contextlib.nested(self.firewall_rule(name='fwr1',
                                                  no_delete=True)) as fr:
            fw_rule_ids = [r['firewall_rule']['id'] for r in fr]
            with self.firewall_policy(firewall_rules=fw_rule_ids,
                                      no_delete=True) as fwp:
                fwp_id = fwp['firewall_policy']['id']
                with self.firewall(name=name,
                                   firewall_policy_id=fwp_id) as firewall:
                    fw_create = firewall['firewall']
                    fw_create = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_create)

                    data = {'firewall_rule': {'name': name,
                                              'source_port': '10:20',
                                              'destination_port': '30:40'}}
                    req = self.new_update_request(
                        'firewall_rules', data,
                        fr[0]['firewall_rule']['id'])
                    res = self.deserialize(self.fmt,
                                           req.get_response(self.ext_api))
                    rule_expect = res['firewall_rule']
                    rule_expect['edge_id'] = VSE_ID
                    self.driver.update_firewall_rule(
                        ctx, rule_expect['id'], VSE_ID, rule_expect)
                    rule_get = self.driver.get_firewall_rule(
                        ctx, rule_expect['id'], VSE_ID)
                    for k, v in rule_get['firewall_rule'].items():
                        self.assertEqual(rule_expect[k], v)

    def test_delete_firewall_rule(self):
        ctx = context.get_admin_context()
        name = 'new_firewall'
        with contextlib.nested(self.firewall_rule(name='fwr1',
                                                  no_delete=True),
                               self.firewall_rule(name='fwr2',
                                                  no_delete=True)) as fr:
            fw_rule_ids = [r['firewall_rule']['id'] for r in fr]
            with self.firewall_policy(firewall_rules=fw_rule_ids,
                                      no_delete=True) as fwp:
                fwp_id = fwp['firewall_policy']['id']
                with self.firewall(name=name,
                                   firewall_policy_id=fwp_id) as firewall:
                    fw_create = firewall['firewall']
                    fw_create = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_create)

                    fr[0]['firewall_rule']['edge_id'] = VSE_ID
                    self.driver.delete_firewall_rule(
                        ctx, fr[0]['firewall_rule']['id'],
                        VSE_ID)
                    self.assertRaises(vcns_exc.VcnsNotFound,
                                      self.driver.get_firewall_rule,
                                      ctx, fr[0]['firewall_rule']['id'],
                                      VSE_ID)

    def test_insert_rule(self):
        ctx = context.get_admin_context()
        with self.firewall_policy() as fwp:
            fwp_id = fwp['firewall_policy']['id']
            with self.firewall(firewall_policy_id=fwp_id) as firewall:
                fw_create = firewall['firewall']
                fw_create = self._make_firewall_dict_with_rules(
                    ctx, fw_create['id'])
                self.driver.update_firewall(ctx, VSE_ID, fw_create)
                with contextlib.nested(self.firewall_rule(name='fwr0',
                                                          no_delete=True),
                                       self.firewall_rule(name='fwr1',
                                                          no_delete=True),
                                       self.firewall_rule(name='fwr2',
                                                          no_delete=True),
                                       self.firewall_rule(name='fwr3',
                                                          no_delete=True),
                                       self.firewall_rule(name='fwr4',
                                                          no_delete=True),
                                       self.firewall_rule(name='fwr5',
                                                          no_delete=True),
                                       self.firewall_rule(
                                           name='fwr6',
                                           no_delete=True)) as fwr:
                    # test insert when rule list is empty
                    fwr0_id = fwr[0]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr0_id,
                                      insert_before=None,
                                      insert_after=None,
                                      expected_code=webob.exc.HTTPOk.code)
                    fw_update = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])
                    self.driver.update_firewall(ctx, VSE_ID, fw_update)
                    # test insert at top of list above existing rule
                    fwr1_id = fwr[1]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr1_id,
                                      insert_before=fwr0_id,
                                      insert_after=None,
                                      expected_code=webob.exc.HTTPOk.code)

                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])

                    rule_info = {'firewall_rule_id': fwr1_id,
                                 'insert_before': fwr0_id,
                                 'insert_after': None}
                    rule = fwr[1]['firewall_rule']
                    self.driver.insert_rule(ctx, rule_info, VSE_ID, rule)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])
                    # test insert at bottom of list
                    fwr2_id = fwr[2]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr2_id,
                                      insert_before=None,
                                      insert_after=fwr0_id,
                                      expected_code=webob.exc.HTTPOk.code)
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])

                    rule_info = {'firewall_rule_id': fwr2_id,
                                 'insert_before': None,
                                 'insert_after': fwr0_id}
                    rule = fwr[2]['firewall_rule']
                    self.driver.insert_rule(ctx, rule_info, VSE_ID, rule)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])
                    # test insert in the middle of the list using
                    # insert_before
                    fwr3_id = fwr[3]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr3_id,
                                      insert_before=fwr2_id,
                                      insert_after=None,
                                      expected_code=webob.exc.HTTPOk.code)
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])

                    rule_info = {'firewall_rule_id': fwr3_id,
                                 'insert_before': fwr2_id,
                                 'insert_after': None}
                    rule = fwr[3]['firewall_rule']
                    self.driver.insert_rule(ctx, rule_info, VSE_ID, rule)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])
                    # test insert in the middle of the list using
                    # insert_after
                    fwr4_id = fwr[4]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr4_id,
                                      insert_before=None,
                                      insert_after=fwr3_id,
                                      expected_code=webob.exc.HTTPOk.code)
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])

                    rule_info = {'firewall_rule_id': fwr4_id,
                                 'insert_before': None,
                                 'insert_after': fwr3_id}
                    rule = fwr[4]['firewall_rule']
                    self.driver.insert_rule(ctx, rule_info, VSE_ID, rule)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])
                    # test insert when both insert_before and
                    # insert_after are set
                    fwr5_id = fwr[5]['firewall_rule']['id']
                    self._rule_action('insert', fwp_id, fwr5_id,
                                      insert_before=fwr4_id,
                                      insert_after=fwr4_id,
                                      expected_code=webob.exc.HTTPOk.code)
                    fw_expect = self._make_firewall_dict_with_rules(
                        ctx, fw_create['id'])

                    rule_info = {'firewall_rule_id': fwr5_id,
                                 'insert_before': fwr4_id,
                                 'insert_after': fwr4_id}
                    rule = fwr[5]['firewall_rule']
                    self.driver.insert_rule(ctx, rule_info, VSE_ID, rule)
                    fw_get = self.driver.get_firewall(
                        ctx, VSE_ID)
                    self._compare_firewall_rule_lists(
                        fwp_id, fw_get['firewall_rule_list'],
                        fw_expect['firewall_rule_list'])



# Copyright 2014 VMware, Inc.

# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""NSX DHCP/metadata support

Revision ID: 1421183d533f
Revises: 8f682276ee4
Create Date: 2013-10-11 14:33:37.303215

"""

revision = '1421183d533f'
down_revision = '8f682276ee4'

migration_for_plugins = [
    'neutron.plugins.nicira.NeutronPlugin.NvpPluginV2',
    'neutron.plugins.nicira.NeutronServicePlugin.NvpAdvancedPlugin',
    'neutron.plugins.vmware.plugin.NsxPlugin',
    'neutron.plugins.vmware.plugin.NsxServicePlugin'
]

from alembic import op
import sqlalchemy as sa

from neutron.db import migration


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.create_table(
        'lsn',
        sa.Column('net_id',
                  sa.String(length=36), nullable=False),
        sa.Column('lsn_id',
                  sa.String(length=36), nullable=False),
        sa.PrimaryKeyConstraint('lsn_id'))

    op.create_table(
        'lsn_port',
        sa.Column('lsn_port_id',
                  sa.String(length=36), nullable=False),
        sa.Column('lsn_id',
                  sa.String(length=36), nullable=False),
        sa.Column('sub_id',
                  sa.String(length=36), nullable=False, unique=True),
        sa.Column('mac_addr',
                  sa.String(length=32), nullable=False, unique=True),
        sa.ForeignKeyConstraint(['lsn_id'], ['lsn.lsn_id'],
                                ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('lsn_port_id'))


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.drop_table('lsn_port')
    op.drop_table('lsn')



# Copyright (c) 2013 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from contextlib import nested
import httplib

import mock

from neutron.openstack.common import importutils
from neutron.tests import base
from neutron.tests.unit.ryu import fake_ryu


class RyuAgentTestCase(base.BaseTestCase):

    _AGENT_NAME = 'neutron.plugins.ryu.agent.ryu_neutron_agent'

    def setUp(self):
        super(RyuAgentTestCase, self).setUp()
        self.fake_ryu = fake_ryu.patch_fake_ryu_client().start()
        self.mod_agent = importutils.import_module(self._AGENT_NAME)


class TestOVSNeutronOFPRyuAgent(RyuAgentTestCase):
    def setUp(self):
        super(TestOVSNeutronOFPRyuAgent, self).setUp()
        self.plugin_api = mock.patch(
            self._AGENT_NAME + '.RyuPluginApi').start()
        self.ovsbridge = mock.patch(
            self._AGENT_NAME + '.OVSBridge').start()
        self.vifportset = mock.patch(
            self._AGENT_NAME + '.VifPortSet').start()
        self.q_ctx = mock.patch(
            self._AGENT_NAME + '.q_context').start()
        self.agent_rpc = mock.patch(
            self._AGENT_NAME + '.agent_rpc.create_consumers').start()
        self.sg_rpc = mock.patch(
            self._AGENT_NAME + '.sg_rpc').start()
        self.sg_agent = mock.patch(
            self._AGENT_NAME + '.RyuSecurityGroupAgent').start()

    def mock_rest_addr(self, rest_addr):
        integ_br = 'integ_br'
        tunnel_ip = '192.168.0.1'
        ovsdb_ip = '172.16.0.1'
        ovsdb_port = 16634
        interval = 2
        root_helper = 'helper'

        self.mod_agent.OVSBridge.return_value.datapath_id = '1234'

        mock_context = mock.Mock(return_value='abc')
        self.q_ctx.get_admin_context_without_session = mock_context

        mock_rest_addr = mock.Mock(return_value=rest_addr)
        self.plugin_api.return_value.get_ofp_rest_api_addr = mock_rest_addr

        # Instantiate OVSNeutronOFPRyuAgent
        return self.mod_agent.OVSNeutronOFPRyuAgent(
            integ_br, tunnel_ip, ovsdb_ip, ovsdb_port, interval, root_helper)

    def test_valid_rest_addr(self):
        self.mock_rest_addr('192.168.0.1:8080')

        # OVSBridge
        self.ovsbridge.assert_has_calls([
            mock.call('integ_br', 'helper'),
            mock.call().find_datapath_id()
        ])

        # RyuPluginRpc
        self.plugin_api.assert_has_calls([
            mock.call('q-plugin'),
            mock.call().get_ofp_rest_api_addr('abc')
        ])

        # Agent RPC
        self.agent_rpc.assert_has_calls([
            mock.call(mock.ANY, 'q-agent-notifier', mock.ANY)
        ])

        # OFPClient
        self.mod_agent.client.OFPClient.assert_has_calls([
            mock.call('192.168.0.1:8080')
        ])

        # VifPortSet
        self.vifportset.assert_has_calls([
            mock.call(
                self.ovsbridge.return_value,
                self.mod_agent.client.OFPClient.return_value),
            mock.call().setup()
        ])

        # SwitchConfClient
        self.mod_agent.client.SwitchConfClient.assert_has_calls([
            mock.call('192.168.0.1:8080'),
            mock.call().set_key('1234', 'ovs_tunnel_addr', '192.168.0.1'),
            mock.call().set_key('1234', 'ovsdb_addr',
                                'tcp:%s:%d' % ('172.16.0.1', 16634))
        ])

        # OVSBridge
        self.ovsbridge.return_value.set_manager.assert_has_calls([
            mock.call('ptcp:%d' % 16634)
        ])

    def test_invalid_rest_addr(self):
        self.assertRaises(self.mod_agent.n_exc.Invalid,
                          self.mock_rest_addr, (''))

    def mock_port_update(self, **kwargs):
        agent = self.mock_rest_addr('192.168.0.1:8080')
        agent.port_update(mock.Mock(), **kwargs)

    def test_port_update(self, **kwargs):
        port = {'id': 1, 'security_groups': 'default'}

        with mock.patch.object(self.ovsbridge.return_value,
                               'get_vif_port_by_id',
                               return_value=1) as get_vif:
            self.mock_port_update(port=port)

        get_vif.assert_called_once_with(1)
        self.sg_agent.assert_has_calls([
            mock.call().refresh_firewall()
        ])

    def test_port_update_not_vifport(self, **kwargs):
        port = {'id': 1, 'security_groups': 'default'}

        with mock.patch.object(self.ovsbridge.return_value,
                               'get_vif_port_by_id',
                               return_value=0) as get_vif:
            self.mock_port_update(port=port)

        get_vif.assert_called_once_with(1)
        self.assertFalse(self.sg_agent.return_value.refresh_firewall.called)

    def test_port_update_without_secgroup(self, **kwargs):
        port = {'id': 1}

        with mock.patch.object(self.ovsbridge.return_value,
                               'get_vif_port_by_id',
                               return_value=1) as get_vif:
            self.mock_port_update(port=port)

        get_vif.assert_called_once_with(1)
        self.assertFalse(self.sg_agent.return_value.refresh_firewall.called)

    def mock_update_ports(self, vif_port_set=None, registered_ports=None):
        with mock.patch.object(self.ovsbridge.return_value,
                               'get_vif_port_set',
                               return_value=vif_port_set):
            agent = self.mock_rest_addr('192.168.0.1:8080')
            return agent._update_ports(registered_ports)

    def test_update_ports_unchanged(self):
        self.assertIsNone(self.mock_update_ports())

    def test_update_ports_changed(self):
        vif_port_set = set([1, 3])
        registered_ports = set([1, 2])
        expected = dict(current=vif_port_set,
                        added=set([3]),
                        removed=set([2]))

        actual = self.mock_update_ports(vif_port_set, registered_ports)

        self.assertEqual(expected, actual)

    def mock_process_devices_filter(self, port_info):
        agent = self.mock_rest_addr('192.168.0.1:8080')
        agent._process_devices_filter(port_info)

    def test_process_devices_filter_add(self):
        port_info = {'added': 1}

        self.mock_process_devices_filter(port_info)

        self.sg_agent.assert_has_calls([
            mock.call().prepare_devices_filter(1)
        ])

    def test_process_devices_filter_remove(self):
        port_info = {'removed': 2}

        self.mock_process_devices_filter(port_info)

        self.sg_agent.assert_has_calls([
            mock.call().remove_devices_filter(2)
        ])

    def test_process_devices_filter_both(self):
        port_info = {'added': 1, 'removed': 2}

        self.mock_process_devices_filter(port_info)

        self.sg_agent.assert_has_calls([
            mock.call().prepare_devices_filter(1),
            mock.call().remove_devices_filter(2)
        ])

    def test_process_devices_filter_none(self):
        port_info = {}

        self.mock_process_devices_filter(port_info)

        self.assertFalse(
            self.sg_agent.return_value.prepare_devices_filter.called)
        self.assertFalse(
            self.sg_agent.return_value.remove_devices_filter.called)


class TestRyuPluginApi(RyuAgentTestCase):
    def test_get_ofp_rest_api_addr(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.RyuPluginApi.make_msg',
                       return_value='msg'),
            mock.patch(self._AGENT_NAME + '.RyuPluginApi.call',
                       return_value='10.0.0.1')
        ) as (mock_msg, mock_call):
            api = self.mod_agent.RyuPluginApi('topics')
            addr = api.get_ofp_rest_api_addr('context')

        self.assertEqual(addr, '10.0.0.1')
        mock_msg.assert_has_calls([
            mock.call('get_ofp_rest_api')
        ])
        mock_call.assert_has_calls([
            mock.call('context', 'msg', topic='topics')
        ])


class TestVifPortSet(RyuAgentTestCase):
    def test_setup(self):
        attrs = {'switch.datapath_id': 'dp1', 'ofport': 'p1'}
        p1 = mock.Mock(**attrs)
        attrs = {'switch.datapath_id': 'dp2', 'ofport': 'p2'}
        p2 = mock.Mock(**attrs)
        attrs = {'get_external_ports.return_value': [p1, p2]}
        int_br = mock.Mock(**attrs)
        with mock.patch(self._AGENT_NAME + '.client.OFPClient') as client:
            api = client()
            vif = self.mod_agent.VifPortSet(int_br, api)
            vif.setup()

        client.assert_has_calls([
            mock.call().update_port('__NW_ID_EXTERNAL__', 'dp1', 'p1'),
            mock.call().update_port('__NW_ID_EXTERNAL__', 'dp2', 'p2')
        ])

    def test_setup_empty(self):
        attrs = {'get_external_ports.return_value': []}
        int_br = mock.Mock(**attrs)
        api = mock.Mock()

        vif = self.mod_agent.VifPortSet(int_br, api)
        vif.setup()

        self.assertEqual(api.update_port.call_count, 0)


class TestOVSBridge(RyuAgentTestCase):
    def setUp(self):
        super(TestOVSBridge, self).setUp()
        self.lib_ovs = mock.patch(
            'neutron.agent.linux.ovs_lib.OVSBridge').start()

    def test_find_datapath_id(self):
        with mock.patch(self._AGENT_NAME + '.OVSBridge.get_datapath_id',
                        return_value='1234') as mock_get_dpid:
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            br.find_datapath_id()

        mock_get_dpid.assert_has_calls([
            mock.call()
        ])
        self.assertEqual(br.datapath_id, '1234')

    def test_set_manager(self):
        with mock.patch(
                self._AGENT_NAME + '.OVSBridge.run_vsctl') as mock_vsctl:
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            br.set_manager('target')

        mock_vsctl.assert_has_calls([
            mock.call(['set-manager', 'target'])
        ])

    def test_get_ofport(self):
        with mock.patch(
                self._AGENT_NAME + '.OVSBridge.db_get_val',
                return_value=1) as mock_db:
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            ofport = br.get_ofport('name')

        mock_db.assert_has_calls([
            mock.call('Interface', 'name', 'ofport')
        ])
        self.assertEqual(ofport, 1)

    def test_get_ports(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_port_name_list',
                       return_value=['p1', 'p2']),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       return_value=1)
        ) as (mock_name, mock_ofport):
            get_port = mock.Mock(side_effect=['port1', 'port2'])
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            ports = br._get_ports(get_port)

        mock_name.assert_has_calls([
            mock.call()
        ])
        mock_ofport.assert_has_calls([
            mock.call('p1'),
            mock.call('p2')
        ])
        get_port.assert_has_calls([
            mock.call('p1'),
            mock.call('p2')
        ])
        self.assertEqual(len(ports), 2)
        self.assertEqual(ports, ['port1', 'port2'])

    def test_get_ports_empty(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_port_name_list',
                       return_value=[]),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       return_value=1)
        ) as (mock_name, mock_ofport):
            get_port = mock.Mock(side_effect=['port1', 'port2'])
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            ports = br._get_ports(get_port)

        mock_name.assert_has_calls([
            mock.call()
        ])
        self.assertEqual(mock_ofport.call_count, 0)
        self.assertEqual(get_port.call_count, 0)
        self.assertEqual(len(ports), 0)

    def test_get_ports_invalid_ofport(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_port_name_list',
                       return_value=['p1', 'p2']),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       side_effect=[-1, 1])
        ) as (mock_name, mock_ofport):
            get_port = mock.Mock(side_effect=['port1', 'port2'])
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            ports = br._get_ports(get_port)

        mock_name.assert_has_calls([
            mock.call()
        ])
        mock_ofport.assert_has_calls([
            mock.call('p1'),
            mock.call('p2')
        ])
        get_port.assert_has_calls([
            mock.call('p2')
        ])
        self.assertEqual(len(ports), 1)
        self.assertEqual(ports, ['port1'])

    def test_get_ports_invalid_port(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_port_name_list',
                       return_value=['p1', 'p2']),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       side_effect=[1, 2])
        ) as (mock_name, mock_ofport):
            get_port = mock.Mock(side_effect=[None, 'port2'])
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            ports = br._get_ports(get_port)

        mock_name.assert_has_calls([
            mock.call()
        ])
        mock_ofport.assert_has_calls([
            mock.call('p1'),
            mock.call('p2')
        ])
        get_port.assert_has_calls([
            mock.call('p1'),
            mock.call('p2')
        ])
        self.assertEqual(len(ports), 1)
        self.assertEqual(ports, ['port2'])

    def test_get_external_port(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.db_get_map',
                       side_effect=[None, {'opts': 'opts_val'}]),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       return_value=1),
            mock.patch(self._AGENT_NAME + '.VifPort')
        ) as (mock_db, mock_ofport, mock_vif):
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            vifport = br._get_external_port('iface')

        mock_db.assert_has_calls([
            mock.call('Interface', 'iface', 'external_ids'),
            mock.call('Interface', 'iface', 'options'),
        ])
        mock_ofport.assert_has_calls([
            mock.call('iface')
        ])
        mock_vif.assert_has_calls([
            mock.call('iface', 1, None, None, br)
        ])
        self.assertEqual(vifport, mock_vif.return_value)

    def test_get_external_port_vmport(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.db_get_map',
                       side_effect=[{'extids': 'extid_val'},
                                    {'opts': 'opts_val'}]),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       return_value=1),
            mock.patch(self._AGENT_NAME + '.VifPort')
        ) as (mock_db, mock_ofport, mock_vif):
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            vifport = br._get_external_port('iface')

        mock_db.assert_has_calls([
            mock.call('Interface', 'iface', 'external_ids'),
        ])
        self.assertEqual(mock_ofport.call_count, 0)
        self.assertEqual(mock_vif.call_count, 0)
        self.assertIsNone(vifport)

    def test_get_external_port_tunnel(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge.db_get_map',
                       side_effect=[None, {'remote_ip': '0.0.0.0'}]),
            mock.patch(self._AGENT_NAME + '.OVSBridge.get_ofport',
                       return_value=1),
            mock.patch(self._AGENT_NAME + '.VifPort')
        ) as (mock_db, mock_ofport, mock_vif):
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            vifport = br._get_external_port('iface')

        mock_db.assert_has_calls([
            mock.call('Interface', 'iface', 'external_ids'),
            mock.call('Interface', 'iface', 'options'),
        ])
        self.assertEqual(mock_ofport.call_count, 0)
        self.assertEqual(mock_vif.call_count, 0)
        self.assertIsNone(vifport)

    def test_get_external_ports(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSBridge._get_external_port'),
            mock.patch(self._AGENT_NAME + '.OVSBridge._get_ports')
        ) as (mock_extport, mock_port):
            br = self.mod_agent.OVSBridge('br_name', 'helper')
            br.get_external_ports()

        mock_port.assert_has_calls([
            mock.call(mock_extport)
        ])


class TestRyuNeutronAgent(RyuAgentTestCase):
    def test_get_my_ip(self):
        sock_attrs = {
            'return_value.getsockname.return_value': ['1.2.3.4', '']}
        with mock.patch('socket.socket', **sock_attrs):
            addr = self.mod_agent._get_my_ip()

        self.assertEqual(addr, '1.2.3.4')

    def test_get_ip_from_nic(self):
        mock_device = mock.Mock()
        mock_device.addr.list = mock.Mock(
            return_value=[{'ip_version': 6, 'cidr': '::ffff:1.2.3.4'},
                          {'ip_version': 4, 'cidr': '1.2.3.4/8'}])
        mock_ip_wrapper = mock.Mock()
        mock_ip_wrapper.device = mock.Mock(return_value=mock_device)
        with mock.patch(self._AGENT_NAME + '.ip_lib.IPWrapper',
                        return_value=mock_ip_wrapper):
            addr = self.mod_agent._get_ip_from_nic('eth0')

        self.assertEqual(addr, '1.2.3.4')

    def test_get_ip_from_nic_empty(self):
        mock_device = mock.Mock()
        mock_device.addr.list = mock.Mock(return_value=[])
        mock_ip_wrapper = mock.Mock()
        mock_ip_wrapper.device = mock.Mock(return_value=mock_device)
        with mock.patch(self._AGENT_NAME + '.ip_lib.IPWrapper',
                        return_value=mock_ip_wrapper):
            addr = self.mod_agent._get_ip_from_nic('eth0')

        self.assertIsNone(addr)

    def test_get_ip_ip(self):
        cfg_attrs = {'CONF.OVS.cfg_ip': '1.2.3.4',
                     'CONF.OVS.cfg_iface': 'eth0'}
        with nested(
            mock.patch(self._AGENT_NAME + '.cfg', **cfg_attrs),
            mock.patch(self._AGENT_NAME + '._get_ip_from_nic',
                       return_value='10.0.0.1'),
            mock.patch(self._AGENT_NAME + '._get_my_ip',
                       return_value='172.16.0.1')
        ) as (_cfg, mock_nicip, mock_myip):
            ip = self.mod_agent._get_ip('cfg_ip', 'cfg_iface')

        self.assertEqual(mock_nicip.call_count, 0)
        self.assertEqual(mock_myip.call_count, 0)
        self.assertEqual(ip, '1.2.3.4')

    def test_get_ip_nic(self):
        cfg_attrs = {'CONF.OVS.cfg_ip': None,
                     'CONF.OVS.cfg_iface': 'eth0'}
        with nested(
            mock.patch(self._AGENT_NAME + '.cfg', **cfg_attrs),
            mock.patch(self._AGENT_NAME + '._get_ip_from_nic',
                       return_value='10.0.0.1'),
            mock.patch(self._AGENT_NAME + '._get_my_ip',
                       return_value='172.16.0.1')
        ) as (_cfg, mock_nicip, mock_myip):
            ip = self.mod_agent._get_ip('cfg_ip', 'cfg_iface')

        mock_nicip.assert_has_calls([
            mock.call('eth0')
        ])
        self.assertEqual(mock_myip.call_count, 0)
        self.assertEqual(ip, '10.0.0.1')

    def test_get_ip_myip(self):
        cfg_attrs = {'CONF.OVS.cfg_ip': None,
                     'CONF.OVS.cfg_iface': None}
        with nested(
            mock.patch(self._AGENT_NAME + '.cfg', **cfg_attrs),
            mock.patch(self._AGENT_NAME + '._get_ip_from_nic',
                       return_value='10.0.0.1'),
            mock.patch(self._AGENT_NAME + '._get_my_ip',
                       return_value='172.16.0.1')
        ) as (_cfg, mock_nicip, mock_myip):
            ip = self.mod_agent._get_ip('cfg_ip', 'cfg_iface')

        self.assertEqual(mock_nicip.call_count, 0)
        mock_myip.assert_has_calls([
            mock.call()
        ])
        self.assertEqual(ip, '172.16.0.1')

    def test_get_ip_nic_myip(self):
        cfg_attrs = {'CONF.OVS.cfg_ip': None,
                     'CONF.OVS.cfg_iface': 'eth0'}
        with nested(
            mock.patch(self._AGENT_NAME + '.cfg', **cfg_attrs),
            mock.patch(self._AGENT_NAME + '._get_ip_from_nic',
                       return_value=None),
            mock.patch(self._AGENT_NAME + '._get_my_ip',
                       return_value='172.16.0.1')
        ) as (_cfg, mock_nicip, mock_myip):
            ip = self.mod_agent._get_ip('cfg_ip', 'cfg_iface')

        mock_nicip.assert_has_calls([
            mock.call('eth0')
        ])
        mock_myip.assert_has_calls([
            mock.call()
        ])
        self.assertEqual(ip, '172.16.0.1')

    def test_get_tunnel_ip(self):
        with mock.patch(self._AGENT_NAME + '._get_ip',
                        return_value='1.2.3.4') as mock_getip:
            ip = self.mod_agent._get_tunnel_ip()

        mock_getip.assert_has_calls([
            mock.call('tunnel_ip', 'tunnel_interface')
        ])
        self.assertEqual(ip, '1.2.3.4')

    def test_get_ovsdb_ip(self):
        with mock.patch(self._AGENT_NAME + '._get_ip',
                        return_value='1.2.3.4') as mock_getip:
            ip = self.mod_agent._get_ovsdb_ip()

        mock_getip.assert_has_calls([
            mock.call('ovsdb_ip', 'ovsdb_interface')
        ])
        self.assertEqual(ip, '1.2.3.4')

    def mock_main(self):
        cfg_attrs = {'CONF.OVS.integration_bridge': 'integ_br',
                     'CONF.OVS.ovsdb_port': 16634,
                     'CONF.AGENT.polling_interval': 2,
                     'CONF.AGENT.root_helper': 'helper'}
        with nested(
            mock.patch(self._AGENT_NAME + '.cfg', **cfg_attrs),
            mock.patch(self._AGENT_NAME + '.logging_config'),
            mock.patch(self._AGENT_NAME + '._get_tunnel_ip',
                       return_value='10.0.0.1'),
            mock.patch(self._AGENT_NAME + '._get_ovsdb_ip',
                       return_value='172.16.0.1'),
        ) as (mock_conf, mock_log_conf, _tun, _ovsdb):
            self.mod_agent.main()

        mock_log_conf.assert_has_calls([
            mock.call(mock_conf)
        ])

    def test_main(self):
        agent_attrs = {'daemon_loop.side_effect': SystemExit(0)}
        with mock.patch(self._AGENT_NAME + '.OVSNeutronOFPRyuAgent',
                        **agent_attrs) as mock_agent:
            self.assertRaises(SystemExit, self.mock_main)

        mock_agent.assert_has_calls([
            mock.call('integ_br', '10.0.0.1', '172.16.0.1', 16634, 2,
                      'helper'),
            mock.call().daemon_loop()
        ])

    def test_main_raise(self):
        with nested(
            mock.patch(self._AGENT_NAME + '.OVSNeutronOFPRyuAgent',
                       side_effect=httplib.HTTPException('boom')),
            mock.patch('sys.exit', side_effect=SystemExit(0))
        ) as (mock_agent, mock_exit):
            self.assertRaises(SystemExit, self.mock_main)

        mock_agent.assert_has_calls([
            mock.call('integ_br', '10.0.0.1', '172.16.0.1', 16634, 2,
                      'helper')
        ])
        mock_exit.assert_has_calls([
            mock.call(1)
        ])



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit Tests for Mellanox RPC (major reuse of linuxbridge rpc unit tests)
"""

import fixtures
from oslo.config import cfg

from neutron.agent import rpc as agent_rpc
from neutron.common import topics
from neutron.openstack.common import context
from neutron.plugins.mlnx import agent_notify_api
from neutron.tests import base


class rpcApiTestCase(base.BaseTestCase):

    def _test_mlnx_api(self, rpcapi, topic, method, rpc_method,
                       expected_msg=None, **kwargs):
        ctxt = context.RequestContext('fake_user', 'fake_project')
        expected_retval = 'foo' if method == 'call' else None
        if not expected_msg:
            expected_msg = rpcapi.make_msg(method, **kwargs)
        expected_msg['version'] = rpcapi.BASE_RPC_API_VERSION
        if rpc_method == 'cast' and method == 'run_instance':
            kwargs['call'] = False

        self.fake_args = None
        self.fake_kwargs = None

        def _fake_rpc_method(*args, **kwargs):
            self.fake_args = args
            self.fake_kwargs = kwargs
            if expected_retval:
                return expected_retval

        self.useFixture(fixtures.MonkeyPatch(
            'neutron.openstack.common.rpc.' + rpc_method, _fake_rpc_method))

        retval = getattr(rpcapi, method)(ctxt, **kwargs)

        self.assertEqual(expected_retval, retval)
        expected_args = [ctxt, topic, expected_msg]

        for arg, expected_arg in zip(self.fake_args, expected_args):
            self.assertEqual(expected_arg, arg)

    def test_delete_network(self):
        rpcapi = agent_notify_api.AgentNotifierApi(topics.AGENT)
        self._test_mlnx_api(rpcapi,
                            topics.get_topic_name(topics.AGENT,
                                                  topics.NETWORK,
                                                  topics.DELETE),
                            'network_delete', rpc_method='fanout_cast',
                            network_id='fake_request_spec')

    def test_port_update(self):
        cfg.CONF.set_override('rpc_support_old_agents', False, 'AGENT')
        rpcapi = agent_notify_api.AgentNotifierApi(topics.AGENT)
        expected_msg = rpcapi.make_msg('port_update',
                                       port='fake_port',
                                       network_type='vlan',
                                       physical_network='fake_net',
                                       segmentation_id='fake_vlan_id')
        self._test_mlnx_api(rpcapi,
                            topics.get_topic_name(topics.AGENT,
                                                  topics.PORT,
                                                  topics.UPDATE),
                            'port_update', rpc_method='fanout_cast',
                            expected_msg=expected_msg,
                            port='fake_port',
                            network_type='vlan',
                            physical_network='fake_net',
                            vlan_id='fake_vlan_id')

    def test_port_update_ib(self):
        cfg.CONF.set_override('rpc_support_old_agents', False, 'AGENT')
        rpcapi = agent_notify_api.AgentNotifierApi(topics.AGENT)
        expected_msg = rpcapi.make_msg('port_update',
                                       port='fake_port',
                                       network_type='ib',
                                       physical_network='fake_net',
                                       segmentation_id='fake_vlan_id')
        self._test_mlnx_api(rpcapi,
                            topics.get_topic_name(topics.AGENT,
                                                  topics.PORT,
                                                  topics.UPDATE),
                            'port_update', rpc_method='fanout_cast',
                            expected_msg=expected_msg,
                            port='fake_port',
                            network_type='ib',
                            physical_network='fake_net',
                            vlan_id='fake_vlan_id')

    def test_port_update_old_agent(self):
        cfg.CONF.set_override('rpc_support_old_agents', True, 'AGENT')
        rpcapi = agent_notify_api.AgentNotifierApi(topics.AGENT)
        expected_msg = rpcapi.make_msg('port_update',
                                       port='fake_port',
                                       network_type='vlan',
                                       physical_network='fake_net',
                                       segmentation_id='fake_vlan_id',
                                       vlan_id='fake_vlan_id')
        self._test_mlnx_api(rpcapi,
                            topics.get_topic_name(topics.AGENT,
                                                  topics.PORT,
                                                  topics.UPDATE),
                            'port_update', rpc_method='fanout_cast',
                            expected_msg=expected_msg,
                            port='fake_port',
                            network_type='vlan',
                            physical_network='fake_net',
                            vlan_id='fake_vlan_id')

    def test_device_details(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_mlnx_api(rpcapi, topics.PLUGIN,
                            'get_device_details', rpc_method='call',
                            device='fake_device',
                            agent_id='fake_agent_id')

    def test_update_device_down(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_mlnx_api(rpcapi, topics.PLUGIN,
                            'update_device_down', rpc_method='call',
                            device='fake_device',
                            agent_id='fake_agent_id',
                            host='fake_host')

    def test_update_device_up(self):
        rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
        self._test_mlnx_api(rpcapi, topics.PLUGIN,
                            'update_device_up', rpc_method='call',
                            device='fake_device',
                            agent_id='fake_agent_id',
                            host='fake_host')



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""grizzly

Revision ID: grizzly
Revises: 1341ed32cc1e
Create Date: 2013-03-12 23:59:59.000000

"""

# revision identifiers, used by Alembic.
revision = 'grizzly'
down_revision = '1341ed32cc1e'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = ['*']


def upgrade(active_plugins=None, options=None):
    """A no-op migration for marking the Grizzly release."""
    pass


def downgrade(active_plugins=None, options=None):
    """A no-op migration for marking the Grizzly release."""
    pass



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 New Dream Network, LLC (DreamHost)
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Mark McClain, DreamHost

import contextlib

import mock

from neutron.common import exceptions
from neutron.services.loadbalancer.drivers.haproxy import (
    namespace_driver
)
from neutron.tests import base


class TestHaproxyNSDriver(base.BaseTestCase):
    def setUp(self):
        super(TestHaproxyNSDriver, self).setUp()

        conf = mock.Mock()
        conf.haproxy.loadbalancer_state_path = '/the/path'
        conf.interface_driver = 'intdriver'
        conf.haproxy.user_group = 'test_group'
        conf.AGENT.root_helper = 'sudo_test'
        self.mock_importer = mock.patch.object(namespace_driver,
                                               'importutils').start()

        self.rpc_mock = mock.Mock()
        self.driver = namespace_driver.HaproxyNSDriver(
            conf,
            self.rpc_mock
        )
        self.vif_driver = mock.Mock()
        self.driver.vif_driver = self.vif_driver

        self.fake_config = {
            'pool': {'id': 'pool_id', 'status': 'ACTIVE',
                     'admin_state_up': True},
            'vip': {'id': 'vip_id', 'port': {'id': 'port_id'},
                    'status': 'ACTIVE', 'admin_state_up': True}
        }

    def test_get_name(self):
        self.assertEqual(self.driver.get_name(), namespace_driver.DRIVER_NAME)

    def test_create(self):
        with mock.patch.object(self.driver, '_plug') as plug:
            with mock.patch.object(self.driver, '_spawn') as spawn:
                self.driver.create(self.fake_config)

                plug.assert_called_once_with(
                    'qlbaas-pool_id', {'id': 'port_id'}
                )
                spawn.assert_called_once_with(self.fake_config)

    def test_update(self):
        with contextlib.nested(
            mock.patch.object(self.driver, '_get_state_file_path'),
            mock.patch.object(self.driver, '_spawn'),
            mock.patch('__builtin__.open')
        ) as (gsp, spawn, mock_open):
            mock_open.return_value = ['5']

            self.driver.update(self.fake_config)

            mock_open.assert_called_once_with(gsp.return_value, 'r')
            spawn.assert_called_once_with(self.fake_config, ['-sf', '5'])

    def test_spawn(self):
        with contextlib.nested(
            mock.patch.object(namespace_driver.hacfg, 'save_config'),
            mock.patch.object(self.driver, '_get_state_file_path'),
            mock.patch('neutron.agent.linux.ip_lib.IPWrapper')
        ) as (mock_save, gsp, ip_wrap):
            gsp.side_effect = lambda x, y: y

            self.driver._spawn(self.fake_config)

            mock_save.assert_called_once_with('conf', self.fake_config,
                                              'sock', 'test_group')
            cmd = ['haproxy', '-f', 'conf', '-p', 'pid']
            ip_wrap.assert_has_calls([
                mock.call('sudo_test', 'qlbaas-pool_id'),
                mock.call().netns.execute(cmd)
            ])

    def test_undeploy_instance(self):
        with contextlib.nested(
            mock.patch.object(self.driver, '_get_state_file_path'),
            mock.patch.object(namespace_driver, 'kill_pids_in_file'),
            mock.patch.object(self.driver, '_unplug'),
            mock.patch('neutron.agent.linux.ip_lib.IPWrapper'),
            mock.patch('os.path.isdir'),
            mock.patch('shutil.rmtree')
        ) as (gsp, kill, unplug, ip_wrap, isdir, rmtree):
            gsp.side_effect = lambda x, y: '/pool/' + y

            self.driver.pool_to_port_id['pool_id'] = 'port_id'
            isdir.return_value = True

            self.driver.undeploy_instance('pool_id')

            kill.assert_called_once_with('sudo_test', '/pool/pid')
            unplug.assert_called_once_with('qlbaas-pool_id', 'port_id')
            isdir.assert_called_once_with('/pool')
            rmtree.assert_called_once_with('/pool')
            ip_wrap.assert_has_calls([
                mock.call('sudo_test', 'qlbaas-pool_id'),
                mock.call().garbage_collect_namespace()
            ])

    def test_exists(self):
        with contextlib.nested(
            mock.patch.object(self.driver, '_get_state_file_path'),
            mock.patch('neutron.agent.linux.ip_lib.IPWrapper'),
            mock.patch('socket.socket'),
            mock.patch('os.path.exists'),
        ) as (gsp, ip_wrap, socket, path_exists):
            gsp.side_effect = lambda x, y: '/pool/' + y

            ip_wrap.return_value.netns.exists.return_value = True
            path_exists.return_value = True

            self.driver.exists('pool_id')

            ip_wrap.assert_has_calls([
                mock.call('sudo_test'),
                mock.call().netns.exists('qlbaas-pool_id')
            ])

            self.assertTrue(self.driver.exists('pool_id'))

    def test_get_stats(self):
        raw_stats = ('# pxname,svname,qcur,qmax,scur,smax,slim,stot,bin,bout,'
                     'dreq,dresp,ereq,econ,eresp,wretr,wredis,status,weight,'
                     'act,bck,chkfail,chkdown,lastchg,downtime,qlimit,pid,iid,'
                     'sid,throttle,lbtot,tracked,type,rate,rate_lim,rate_max,'
                     'check_status,check_code,check_duration,hrsp_1xx,'
                     'hrsp_2xx,hrsp_3xx,hrsp_4xx,hrsp_5xx,hrsp_other,hanafail,'
                     'req_rate,req_rate_max,req_tot,cli_abrt,srv_abrt,\n'
                     '8e271901-69ed-403e-a59b-f53cf77ef208,BACKEND,1,2,3,4,0,'
                     '10,7764,2365,0,0,,0,0,0,0,UP,1,1,0,,0,103780,0,,1,2,0,,0'
                     ',,1,0,,0,,,,0,0,0,0,0,0,,,,,0,0,\n\n'
                     'a557019b-dc07-4688-9af4-f5cf02bb6d4b,'
                     '32a6c2a3-420a-44c3-955d-86bd2fc6871e,0,0,0,1,,7,1120,'
                     '224,,0,,0,0,0,0,UP,1,1,0,0,1,2623,303,,1,2,1,,7,,2,0,,'
                     '1,L7OK,200,98,0,7,0,0,0,0,0,,,,0,0,\n'
                     'a557019b-dc07-4688-9af4-f5cf02bb6d4b,'
                     'd9aea044-8867-4e80-9875-16fb808fa0f9,0,0,0,2,,12,0,0,,'
                     '0,,0,0,8,4,DOWN,1,1,0,9,2,308,675,,1,2,2,,4,,2,0,,2,'
                     'L4CON,,2999,0,0,0,0,0,0,0,,,,0,0,\n')
        raw_stats_empty = ('# pxname,svname,qcur,qmax,scur,smax,slim,stot,bin,'
                           'bout,dreq,dresp,ereq,econ,eresp,wretr,wredis,'
                           'status,weight,act,bck,chkfail,chkdown,lastchg,'
                           'downtime,qlimit,pid,iid,sid,throttle,lbtot,'
                           'tracked,type,rate,rate_lim,rate_max,check_status,'
                           'check_code,check_duration,hrsp_1xx,hrsp_2xx,'
                           'hrsp_3xx,hrsp_4xx,hrsp_5xx,hrsp_other,hanafail,'
                           'req_rate,req_rate_max,req_tot,cli_abrt,srv_abrt,'
                           '\n')
        with contextlib.nested(
                mock.patch.object(self.driver, '_get_state_file_path'),
                mock.patch('socket.socket'),
                mock.patch('os.path.exists'),
        ) as (gsp, socket, path_exists):
            gsp.side_effect = lambda x, y: '/pool/' + y
            path_exists.return_value = True
            socket.return_value = socket
            socket.recv.return_value = raw_stats

            exp_stats = {'connection_errors': '0',
                         'active_connections': '1',
                         'current_sessions': '3',
                         'bytes_in': '7764',
                         'max_connections': '2',
                         'max_sessions': '4',
                         'bytes_out': '2365',
                         'response_errors': '0',
                         'total_sessions': '10',
                         'members': {
                             '32a6c2a3-420a-44c3-955d-86bd2fc6871e': {
                                 'status': 'ACTIVE',
                                 'health': 'L7OK',
                                 'failed_checks': '0'
                             },
                             'd9aea044-8867-4e80-9875-16fb808fa0f9': {
                                 'status': 'INACTIVE',
                                 'health': 'L4CON',
                                 'failed_checks': '9'
                             }
                         }
                         }
            stats = self.driver.get_stats('pool_id')
            self.assertEqual(exp_stats, stats)

            socket.recv.return_value = raw_stats_empty
            self.assertEqual({'members': {}}, self.driver.get_stats('pool_id'))

            path_exists.return_value = False
            socket.reset_mock()
            self.assertEqual({}, self.driver.get_stats('pool_id'))
            self.assertFalse(socket.called)

    def test_plug(self):
        test_port = {'id': 'port_id',
                     'network_id': 'net_id',
                     'mac_address': 'mac_addr',
                     'fixed_ips': [{'ip_address': '10.0.0.2',
                                    'subnet': {'cidr': '10.0.0.0/24',
                                               'gateway_ip': '10.0.0.1'}}]}
        with contextlib.nested(
                mock.patch('neutron.agent.linux.ip_lib.device_exists'),
                mock.patch('netaddr.IPNetwork'),
                mock.patch('neutron.agent.linux.ip_lib.IPWrapper'),
        ) as (dev_exists, ip_net, ip_wrap):
            self.vif_driver.get_device_name.return_value = 'test_interface'
            dev_exists.return_value = False
            ip_net.return_value = ip_net
            ip_net.prefixlen = 24

            self.driver._plug('test_ns', test_port)
            self.rpc_mock.plug_vip_port.assert_called_once_with(
                test_port['id'])
            self.assertTrue(dev_exists.called)
            self.vif_driver.plug.assert_called_once_with('net_id', 'port_id',
                                                         'test_interface',
                                                         'mac_addr',
                                                         namespace='test_ns')
            self.vif_driver.init_l3.assert_called_once_with('test_interface',
                                                            ['10.0.0.2/24'],
                                                            namespace=
                                                            'test_ns')
            cmd = ['route', 'add', 'default', 'gw', '10.0.0.1']
            ip_wrap.assert_has_calls([
                mock.call('sudo_test', namespace='test_ns'),
                mock.call().netns.execute(cmd, check_exit_code=False),
            ])

            dev_exists.return_value = True
            self.assertRaises(exceptions.PreexistingDeviceFailure,
                              self.driver._plug, 'test_ns', test_port, False)

    def test_plug_no_gw(self):
        test_port = {'id': 'port_id',
                     'network_id': 'net_id',
                     'mac_address': 'mac_addr',
                     'fixed_ips': [{'ip_address': '10.0.0.2',
                                    'subnet': {'cidr': '10.0.0.0/24'}}]}
        with contextlib.nested(
                mock.patch('neutron.agent.linux.ip_lib.device_exists'),
                mock.patch('netaddr.IPNetwork'),
                mock.patch('neutron.agent.linux.ip_lib.IPWrapper'),
        ) as (dev_exists, ip_net, ip_wrap):
            self.vif_driver.get_device_name.return_value = 'test_interface'
            dev_exists.return_value = False
            ip_net.return_value = ip_net
            ip_net.prefixlen = 24

            self.driver._plug('test_ns', test_port)
            self.rpc_mock.plug_vip_port.assert_called_once_with(
                test_port['id'])
            self.assertTrue(dev_exists.called)
            self.vif_driver.plug.assert_called_once_with('net_id', 'port_id',
                                                         'test_interface',
                                                         'mac_addr',
                                                         namespace='test_ns')
            self.vif_driver.init_l3.assert_called_once_with('test_interface',
                                                            ['10.0.0.2/24'],
                                                            namespace=
                                                            'test_ns')
            self.assertFalse(ip_wrap.called)
            dev_exists.return_value = True
            self.assertRaises(exceptions.PreexistingDeviceFailure,
                              self.driver._plug, 'test_ns', test_port, False)

    def test_unplug(self):
        self.vif_driver.get_device_name.return_value = 'test_interface'

        self.driver._unplug('test_ns', 'port_id')
        self.rpc_mock.unplug_vip_port.assert_called_once_with('port_id')
        self.vif_driver.unplug('test_interface', namespace='test_ns')

    def test_kill_pids_in_file(self):
        with contextlib.nested(
            mock.patch('os.path.exists'),
            mock.patch('__builtin__.open'),
            mock.patch('neutron.agent.linux.utils.execute'),
            mock.patch.object(namespace_driver.LOG, 'exception'),
        ) as (path_exists, mock_open, mock_execute, mock_log):
            file_mock = mock.MagicMock()
            mock_open.return_value = file_mock
            file_mock.__enter__.return_value = file_mock
            file_mock.__iter__.return_value = iter(['123'])

            path_exists.return_value = False
            namespace_driver.kill_pids_in_file('sudo_test', 'test_path')
            path_exists.assert_called_once_with('test_path')
            self.assertFalse(mock_open.called)
            self.assertFalse(mock_execute.called)

            path_exists.return_value = True
            mock_execute.side_effect = RuntimeError
            namespace_driver.kill_pids_in_file('sudo_test', 'test_path')
            self.assertTrue(mock_log.called)
            mock_execute.assert_called_once_with(
                ['kill', '-9', '123'], 'sudo_test')

    def test_get_state_file_path(self):
        with mock.patch('os.makedirs') as mkdir:
            path = self.driver._get_state_file_path('pool_id', 'conf')
            self.assertEqual('/the/path/pool_id/conf', path)
            mkdir.assert_called_once_with('/the/path/pool_id', 0o755)

    def test_deploy_instance(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            with mock.patch.object(self.driver, 'update') as update:
                self.driver.deploy_instance(self.fake_config)
                exists.assert_called_once_with(self.fake_config['pool']['id'])
                update.assert_called_once_with(self.fake_config)

    def test_deploy_instance_non_existing(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            with mock.patch.object(self.driver, 'create') as create:
                exists.return_value = False
                self.driver.deploy_instance(self.fake_config)
                exists.assert_called_once_with(self.fake_config['pool']['id'])
                create.assert_called_once_with(self.fake_config)

    def test_deploy_instance_vip_status_non_active(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            self.fake_config['vip']['status'] = 'NON_ACTIVE'
            self.driver.deploy_instance(self.fake_config)
            self.assertFalse(exists.called)

    def test_deploy_instance_vip_admin_state_down(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            self.fake_config['vip']['admin_state_up'] = False
            self.driver.deploy_instance(self.fake_config)
            self.assertFalse(exists.called)

    def test_deploy_instance_no_vip(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            del self.fake_config['vip']
            self.driver.deploy_instance(self.fake_config)
            self.assertFalse(exists.called)

    def test_deploy_instance_pool_status_non_active(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            self.fake_config['pool']['status'] = 'NON_ACTIVE'
            self.driver.deploy_instance(self.fake_config)
            self.assertFalse(exists.called)

    def test_deploy_instance_pool_admin_state_down(self):
        with mock.patch.object(self.driver, 'exists') as exists:
            self.fake_config['pool']['admin_state_up'] = False
            self.driver.deploy_instance(self.fake_config)
            self.assertFalse(exists.called)

    def test_refresh_device(self):
        with mock.patch.object(self.driver, 'deploy_instance') as deploy:
            pool_id = 'pool_id1'
            self.driver._refresh_device(pool_id)
            self.rpc_mock.get_logical_device.assert_called_once_with(pool_id)
            deploy.assert_called_once_with(
                self.rpc_mock.get_logical_device.return_value)

    def test_create_vip(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.create_vip({'pool_id': '1'})
            refresh.assert_called_once_with('1')

    def test_update_vip(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.update_vip({}, {'pool_id': '1'})
            refresh.assert_called_once_with('1')

    def test_delete_vip(self):
        with mock.patch.object(self.driver, 'undeploy_instance') as undeploy:
            self.driver.delete_vip({'pool_id': '1'})
            undeploy.assert_called_once_with('1')

    def test_create_pool(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.create_pool({'id': '1'})
            self.assertFalse(refresh.called)

    def test_update_pool(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.update_pool({}, {'id': '1'})
            refresh.assert_called_once_with('1')

    def test_delete_pool_existing(self):
        with mock.patch.object(self.driver, 'undeploy_instance') as undeploy:
            with mock.patch.object(self.driver, 'exists') as exists:
                exists.return_value = True
                self.driver.delete_pool({'id': '1'})
                undeploy.assert_called_once_with('1')

    def test_delete_pool_non_existing(self):
        with mock.patch.object(self.driver, 'undeploy_instance') as undeploy:
            with mock.patch.object(self.driver, 'exists') as exists:
                exists.return_value = False
                self.driver.delete_pool({'id': '1'})
                self.assertFalse(undeploy.called)

    def test_create_member(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.create_member({'pool_id': '1'})
            refresh.assert_called_once_with('1')

    def test_update_member(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.update_member({}, {'pool_id': '1'})
            refresh.assert_called_once_with('1')

    def test_delete_member(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.delete_member({'pool_id': '1'})
            refresh.assert_called_once_with('1')

    def test_create_pool_health_monitor(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.create_pool_health_monitor('', '1')
            refresh.assert_called_once_with('1')

    def test_update_pool_health_monitor(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.update_pool_health_monitor('', '', '1')
            refresh.assert_called_once_with('1')

    def test_delete_pool_health_monitor(self):
        with mock.patch.object(self.driver, '_refresh_device') as refresh:
            self.driver.delete_pool_health_monitor('', '1')
            refresh.assert_called_once_with('1')



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2012 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
# @author: Ryota MIBU
# @author: Akihiro MOTOKI

import netaddr

from neutron.common import utils
from neutron.openstack.common import log as logging
from neutron.plugins.nec.common import config
from neutron.plugins.nec.common import exceptions as nexc
from neutron.plugins.nec.db import api as ndb
from neutron.plugins.nec import drivers


LOG = logging.getLogger(__name__)


class OFCManager(object):
    """This class manages an OpenFlow Controller and map resources.

    This class manage an OpenFlow Controller (OFC) with a driver specified in
    a configuration of this plugin.  This keeps mappings between IDs on Neutron
    and OFC for various entities such as Tenant, Network and Filter.  A Port on
    OFC is identified by a switch ID 'datapath_id' and a port number 'port_no'
    of the switch.  An ID named as 'ofc_*' is used to identify resource on OFC.
    """

    def __init__(self, plugin):
        self.driver = drivers.get_driver(config.OFC.driver)(config.OFC)
        self.plugin = plugin

    def _get_ofc_id(self, context, resource, neutron_id):
        return ndb.get_ofc_id(context.session, resource, neutron_id)

    def _exists_ofc_item(self, context, resource, neutron_id):
        return ndb.exists_ofc_item(context.session, resource, neutron_id)

    def _add_ofc_item(self, context, resource, neutron_id, ofc_id):
        # Ensure a new item is added to the new mapping table
        ndb.add_ofc_item(context.session, resource, neutron_id, ofc_id)

    def _del_ofc_item(self, context, resource, neutron_id):
        ndb.del_ofc_item(context.session, resource, neutron_id)

    def ensure_ofc_tenant(self, context, tenant_id):
        if not self.exists_ofc_tenant(context, tenant_id):
            self.create_ofc_tenant(context, tenant_id)

    def create_ofc_tenant(self, context, tenant_id):
        desc = "ID=%s at OpenStack." % tenant_id
        ofc_tenant_id = self.driver.create_tenant(desc, tenant_id)
        self._add_ofc_item(context, "ofc_tenant", tenant_id, ofc_tenant_id)

    def exists_ofc_tenant(self, context, tenant_id):
        return self._exists_ofc_item(context, "ofc_tenant", tenant_id)

    def delete_ofc_tenant(self, context, tenant_id):
        ofc_tenant_id = self._get_ofc_id(context, "ofc_tenant", tenant_id)
        self.driver.delete_tenant(ofc_tenant_id)
        self._del_ofc_item(context, "ofc_tenant", tenant_id)

    def create_ofc_network(self, context, tenant_id, network_id,
                           network_name=None):
        ofc_tenant_id = self._get_ofc_id(context, "ofc_tenant", tenant_id)
        desc = "ID=%s Name=%s at Neutron." % (network_id, network_name)
        ofc_net_id = self.driver.create_network(ofc_tenant_id, desc,
                                                network_id)
        self._add_ofc_item(context, "ofc_network", network_id, ofc_net_id)

    def exists_ofc_network(self, context, network_id):
        return self._exists_ofc_item(context, "ofc_network", network_id)

    def delete_ofc_network(self, context, network_id, network):
        ofc_net_id = self._get_ofc_id(context, "ofc_network", network_id)
        self.driver.delete_network(ofc_net_id)
        self._del_ofc_item(context, "ofc_network", network_id)

    def create_ofc_port(self, context, port_id, port):
        ofc_net_id = self._get_ofc_id(context, "ofc_network",
                                      port['network_id'])
        portinfo = ndb.get_portinfo(context.session, port_id)
        if not portinfo:
            raise nexc.PortInfoNotFound(id=port_id)

        # Associate packet filters
        filters = self.plugin.get_packet_filters_for_port(context, port)
        if filters is not None:
            params = {'filters': filters}
        else:
            params = {}

        ofc_port_id = self.driver.create_port(ofc_net_id, portinfo, port_id,
                                              **params)
        self._add_ofc_item(context, "ofc_port", port_id, ofc_port_id)

    def exists_ofc_port(self, context, port_id):
        return self._exists_ofc_item(context, "ofc_port", port_id)

    def delete_ofc_port(self, context, port_id, port):
        ofc_port_id = self._get_ofc_id(context, "ofc_port", port_id)
        self.driver.delete_port(ofc_port_id)
        self._del_ofc_item(context, "ofc_port", port_id)

    def create_ofc_packet_filter(self, context, filter_id, filter_dict):
        ofc_net_id = self._get_ofc_id(context, "ofc_network",
                                      filter_dict['network_id'])
        in_port_id = filter_dict.get('in_port')
        portinfo = None
        if in_port_id:
            portinfo = ndb.get_portinfo(context.session, in_port_id)
            if not portinfo:
                raise nexc.PortInfoNotFound(id=in_port_id)

        # Collect ports to be associated with the filter
        apply_ports = ndb.get_active_ports_on_ofc(
            context, filter_dict['network_id'], in_port_id)
        ofc_pf_id = self.driver.create_filter(ofc_net_id,
                                              filter_dict, portinfo, filter_id,
                                              apply_ports)
        self._add_ofc_item(context, "ofc_packet_filter", filter_id, ofc_pf_id)

    def update_ofc_packet_filter(self, context, filter_id, filter_dict):
        ofc_pf_id = self._get_ofc_id(context, "ofc_packet_filter", filter_id)
        ofc_pf_id = self.driver.convert_ofc_filter_id(context, ofc_pf_id)
        self.driver.update_filter(ofc_pf_id, filter_dict)

    def exists_ofc_packet_filter(self, context, filter_id):
        return self._exists_ofc_item(context, "ofc_packet_filter", filter_id)

    def delete_ofc_packet_filter(self, context, filter_id):
        ofc_pf_id = self._get_ofc_id(context, "ofc_packet_filter", filter_id)
        self.driver.delete_filter(ofc_pf_id)
        self._del_ofc_item(context, "ofc_packet_filter", filter_id)

    def create_ofc_router(self, context, tenant_id, router_id, name=None):
        ofc_tenant_id = self._get_ofc_id(context, "ofc_tenant", tenant_id)
        desc = "ID=%s Name=%s at Neutron." % (router_id, name)
        ofc_router_id = self.driver.create_router(ofc_tenant_id, router_id,
                                                  desc)
        self._add_ofc_item(context, "ofc_router", router_id, ofc_router_id)

    def exists_ofc_router(self, context, router_id):
        return self._exists_ofc_item(context, "ofc_router", router_id)

    def delete_ofc_router(self, context, router_id, router):
        ofc_router_id = self._get_ofc_id(context, "ofc_router", router_id)
        self.driver.delete_router(ofc_router_id)
        self._del_ofc_item(context, "ofc_router", router_id)

    def add_ofc_router_interface(self, context, router_id, port_id, port):
        # port must have the following fields:
        #   network_id, cidr, ip_address, mac_address
        ofc_router_id = self._get_ofc_id(context, "ofc_router", router_id)
        ofc_net_id = self._get_ofc_id(context, "ofc_network",
                                      port['network_id'])
        ip_address = '%s/%s' % (port['ip_address'],
                                netaddr.IPNetwork(port['cidr']).prefixlen)
        mac_address = port['mac_address']
        ofc_inf_id = self.driver.add_router_interface(
            ofc_router_id, ofc_net_id, ip_address, mac_address)
        # Use port mapping table to maintain an interface of OFC router
        self._add_ofc_item(context, "ofc_port", port_id, ofc_inf_id)

    def delete_ofc_router_interface(self, context, router_id, port_id):
        # Use port mapping table to maintain an interface of OFC router
        ofc_inf_id = self._get_ofc_id(context, "ofc_port", port_id)
        self.driver.delete_router_interface(ofc_inf_id)
        self._del_ofc_item(context, "ofc_port", port_id)

    def update_ofc_router_route(self, context, router_id, new_routes):
        ofc_router_id = self._get_ofc_id(context, "ofc_router", router_id)
        ofc_routes = self.driver.list_router_routes(ofc_router_id)
        route_dict = {}
        cur_routes = []
        for r in ofc_routes:
            key = ','.join((r['destination'], r['nexthop']))
            route_dict[key] = r['id']
            del r['id']
            cur_routes.append(r)
        added, removed = utils.diff_list_of_dict(cur_routes, new_routes)
        for r in removed:
            key = ','.join((r['destination'], r['nexthop']))
            route_id = route_dict[key]
            self.driver.delete_router_route(route_id)
        for r in added:
            self.driver.add_router_route(ofc_router_id, r['destination'],
                                         r['nexthop'])



# Copyright (c) 2014 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from neutron.common import test_lib
from neutron.plugins.vmware.common import sync
from neutron.tests.unit import test_extension_portsecurity as psec
from neutron.tests.unit.vmware.apiclient import fake
from neutron.tests.unit.vmware import get_fake_conf
from neutron.tests.unit.vmware import NSXAPI_NAME
from neutron.tests.unit.vmware import PLUGIN_NAME
from neutron.tests.unit.vmware import STUBS_PATH


class PortSecurityTestCase(psec.PortSecurityDBTestCase):

    def setUp(self):
        test_lib.test_config['config_files'] = [get_fake_conf('nsx.ini.test')]
        # mock api client
        self.fc = fake.FakeClient(STUBS_PATH)
        self.mock_nsx = mock.patch(NSXAPI_NAME, autospec=True)
        instance = self.mock_nsx.start()
        instance.return_value.login.return_value = "the_cookie"
        # Avoid runs of the synchronizer looping call
        patch_sync = mock.patch.object(sync, '_start_loopingcall')
        patch_sync.start()

        instance.return_value.request.side_effect = self.fc.fake_request
        super(PortSecurityTestCase, self).setUp(PLUGIN_NAME)
        self.addCleanup(self.fc.reset_all)
        self.addCleanup(self.mock_nsx.stop)
        self.addCleanup(patch_sync.stop)


class TestPortSecurity(PortSecurityTestCase, psec.TestPortSecurity):
        pass



# Copyright 2014 OneConvergence, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Kedar Kulkarni, One Convergence, Inc.

"""Intermidiate NVSD Library."""

from neutron.openstack.common import excutils
from neutron.openstack.common import jsonutils as json
from neutron.openstack.common import log as logging
import neutron.plugins.oneconvergence.lib.exception as nvsdexception
from neutron.plugins.oneconvergence.lib import plugin_helper

LOG = logging.getLogger(__name__)

NETWORKS_URI = "/pluginhandler/ocplugin/tenant/%s/lnetwork/"
NETWORK_URI = NETWORKS_URI + "%s"
GET_ALL_NETWORKS = "/pluginhandler/ocplugin/tenant/getallnetworks"

SUBNETS_URI = NETWORK_URI + "/lsubnet/"
SUBNET_URI = SUBNETS_URI + "%s"
GET_ALL_SUBNETS = "/pluginhandler/ocplugin/tenant/getallsubnets"

PORTS_URI = NETWORK_URI + "/lport/"
PORT_URI = PORTS_URI + "%s"

EXT_URI = "/pluginhandler/ocplugin/ext/tenant/%s"
FLOATING_IPS_URI = EXT_URI + "/floatingip/"
FLOATING_IP_URI = FLOATING_IPS_URI + "%s"

ROUTERS_URI = EXT_URI + "/lrouter/"
ROUTER_URI = ROUTERS_URI + "%s"

METHODS = {"POST": "create",
           "PUT": "update",
           "DELETE": "delete",
           "GET": "get"}


class NVSDApi(object):

    def build_error_msg(self, method, resource, tenant_id, resource_id):
        if method == "POST":
            msg = _("Could not create a %(resource)s under tenant "
                    "%(tenant_id)s") % {'resource': resource,
                                        'tenant_id': tenant_id}
        elif resource_id:
            msg = _("Failed to %(method)s %(resource)s "
                    "id=%(resource_id)s") % {'method': METHODS[method],
                                             'resource': resource,
                                             'resource_id': resource_id
                                             }
        else:
            msg = _("Failed to %(method)s %(resource)s") % {
                'method': METHODS[method], 'resource': resource}
        return msg

    def set_connection(self):
        self.nvsdcontroller = plugin_helper.initialize_plugin_helper()
        self.nvsdcontroller.login()

    def send_request(self, method, uri, body=None, resource=None,
                     tenant_id='', resource_id=None):
        """Issue a request to NVSD controller."""

        try:
            result = self.nvsdcontroller.request(method, uri, body=body)
        except nvsdexception.NVSDAPIException as e:
            with excutils.save_and_reraise_exception() as ctxt:
                msg = self.build_error_msg(method, resource, tenant_id,
                                           resource_id)
                LOG.error(msg)
                # Modifying the reason message without disturbing the exception
                # info
                ctxt.value = type(e)(reason=msg)
        return result

    def create_network(self, network):

        tenant_id = network['tenant_id']
        router_external = network['router:external'] is True

        network_obj = {
            "name": network['name'],
            "tenant_id": tenant_id,
            "shared": network['shared'],
            "admin_state_up": network['admin_state_up'],
            "router:external": router_external
        }

        uri = NETWORKS_URI % tenant_id

        response = self.send_request("POST", uri, body=json.dumps(network_obj),
                                     resource='network', tenant_id=tenant_id)

        nvsd_net = response.json()

        LOG.debug(_("Network %(id)s created under tenant %(tenant_id)s"),
                  {'id': nvsd_net['id'], 'tenant_id': tenant_id})

        return nvsd_net

    def update_network(self, network, network_update):

        tenant_id = network['tenant_id']
        network_id = network['id']

        uri = NETWORK_URI % (tenant_id, network_id)

        self.send_request("PUT", uri,
                          body=json.dumps(network_update),
                          resource='network', tenant_id=tenant_id,
                          resource_id=network_id)

        LOG.debug(_("Network %(id)s updated under tenant %(tenant_id)s"),
                  {'id': network_id, 'tenant_id': tenant_id})

    def delete_network(self, network, subnets=[]):

        tenant_id = network['tenant_id']
        network_id = network['id']

        ports = self._get_ports(tenant_id, network_id)

        for port in ports:
            self.delete_port(port['id'], port)

        for subnet in subnets:
            self.delete_subnet(subnet)

        path = NETWORK_URI % (tenant_id, network_id)

        self.send_request("DELETE", path, resource='network',
                          tenant_id=tenant_id, resource_id=network_id)

        LOG.debug(_("Network %(id)s deleted under tenant %(tenant_id)s"),
                  {'id': network_id, 'tenant_id': tenant_id})

    def create_subnet(self, subnet):

        tenant_id = subnet['tenant_id']
        network_id = subnet['network_id']

        uri = SUBNETS_URI % (tenant_id, network_id)

        self.send_request("POST", uri, body=json.dumps(subnet),
                          resource='subnet', tenant_id=tenant_id)

        LOG.debug(_("Subnet %(id)s created under tenant %(tenant_id)s"),
                  {'id': subnet['id'], 'tenant_id': tenant_id})

    def delete_subnet(self, subnet):

        tenant_id = subnet['tenant_id']
        network_id = subnet['network_id']
        subnet_id = subnet['id']

        uri = SUBNET_URI % (tenant_id, network_id, subnet_id)

        self.send_request("DELETE", uri, resource='subnet',
                          tenant_id=tenant_id, resource_id=subnet_id)

        LOG.debug(_("Subnet %(id)s deleted under tenant %(tenant_id)s"),
                  {'id': subnet_id, 'tenant_id': tenant_id})

    def update_subnet(self, subnet, subnet_update):

        tenant_id = subnet['tenant_id']
        network_id = subnet['network_id']
        subnet_id = subnet['id']

        uri = SUBNET_URI % (tenant_id, network_id, subnet_id)

        self.send_request("PUT", uri,
                          body=json.dumps(subnet_update),
                          resource='subnet', tenant_id=tenant_id,
                          resource_id=subnet_id)

        LOG.debug(_("Subnet %(id)s updated under tenant %(tenant_id)s"),
                  {'id': subnet_id, 'tenant_id': tenant_id})

    def create_port(self, tenant_id, port):

        network_id = port["network_id"]
        fixed_ips = port.get("fixed_ips")
        ip_address = None
        subnet_id = None

        if fixed_ips:
            ip_address = fixed_ips[0].get("ip_address")
            subnet_id = fixed_ips[0].get("subnet_id")

        lport = {
            "id": port["id"],
            "name": port["name"],
            "device_id": port["device_id"],
            "device_owner": port["device_owner"],
            "mac_address": port["mac_address"],
            "ip_address": ip_address,
            "subnet_id": subnet_id,
            "admin_state_up": port["admin_state_up"],
            "network_id": network_id,
            "status": port["status"]
        }

        path = PORTS_URI % (tenant_id, network_id)

        self.send_request("POST", path, body=json.dumps(lport),
                          resource='port', tenant_id=tenant_id)

        LOG.debug(_("Port %(id)s created under tenant %(tenant_id)s"),
                  {'id': port['id'], 'tenant_id': tenant_id})

    def update_port(self, tenant_id, port, port_update):

        network_id = port['network_id']
        port_id = port['id']

        lport = {}
        for k in ('admin_state_up', 'name', 'device_id', 'device_owner'):
            if k in port_update:
                lport[k] = port_update[k]

        fixed_ips = port_update.get('fixed_ips', None)
        if fixed_ips:
            lport["ip_address"] = fixed_ips[0].get("ip_address")
            lport["subnet_id"] = fixed_ips[0].get("subnet_id")

        uri = PORT_URI % (tenant_id, network_id, port_id)

        self.send_request("PUT", uri, body=json.dumps(lport),
                          resource='port', tenant_id=tenant_id,
                          resource_id=port_id)

        LOG.debug(_("Port %(id)s updated under tenant %(tenant_id)s"),
                  {'id': port_id, 'tenant_id': tenant_id})

    def delete_port(self, port_id, port):

        tenant_id = port['tenant_id']
        network_id = port['network_id']

        uri = PORT_URI % (tenant_id, network_id, port_id)

        self.send_request("DELETE", uri, resource='port', tenant_id=tenant_id,
                          resource_id=port_id)

        LOG.debug(_("Port %(id)s deleted under tenant %(tenant_id)s"),
                  {'id': port_id, 'tenant_id': tenant_id})

    def _get_ports(self, tenant_id, network_id):

        uri = PORTS_URI % (tenant_id, network_id)

        response = self.send_request("GET", uri, resource='ports',
                                     tenant_id=tenant_id)

        return response.json()

    def create_floatingip(self, floating_ip):

        tenant_id = floating_ip['tenant_id']

        uri = FLOATING_IPS_URI % tenant_id

        self.send_request("POST", uri, body=json.dumps(floating_ip),
                          resource='floating_ip',
                          tenant_id=tenant_id)

        LOG.debug(_("Flatingip %(id)s created under tenant %(tenant_id)s"),
                  {'id': floating_ip['id'], 'tenant_id': tenant_id})

    def update_floatingip(self, floating_ip, floating_ip_update):

        tenant_id = floating_ip['tenant_id']

        floating_ip_id = floating_ip['id']

        uri = FLOATING_IP_URI % (tenant_id, floating_ip_id)

        self.send_request("PUT", uri,
                          body=json.dumps(floating_ip_update['floatingip']),
                          resource='floating_ip',
                          tenant_id=tenant_id,
                          resource_id=floating_ip_id)

        LOG.debug(_("Flatingip %(id)s updated under tenant %(tenant_id)s"),
                  {'id': floating_ip_id, 'tenant_id': tenant_id})

    def delete_floatingip(self, floating_ip):

        tenant_id = floating_ip['tenant_id']

        floating_ip_id = floating_ip['id']

        uri = FLOATING_IP_URI % (tenant_id, floating_ip_id)

        self.send_request("DELETE", uri, resource='floating_ip',
                          tenant_id=tenant_id, resource_id=floating_ip_id)

        LOG.debug(_("Flatingip %(id)s deleted under tenant %(tenant_id)s"),
                  {'id': floating_ip_id, 'tenant_id': tenant_id})

    def create_router(self, router):

        tenant_id = router['tenant_id']

        uri = ROUTERS_URI % tenant_id

        self.send_request("POST", uri, body=json.dumps(router),
                          resource='router',
                          tenant_id=tenant_id)

        LOG.debug(_("Router %(id)s created under tenant %(tenant_id)s"),
                  {'id': router['id'], 'tenant_id': tenant_id})

    def update_router(self, router):

        tenant_id = router['tenant_id']

        router_id = router['id']

        uri = ROUTER_URI % (tenant_id, router_id)

        self.send_request("PUT", uri,
                          body=json.dumps(router),
                          resource='router', tenant_id=tenant_id,
                          resource_id=router_id)

        LOG.debug(_("Router %(id)s updated under tenant %(tenant_id)s"),
                  {'id': router_id, 'tenant_id': tenant_id})

    def delete_router(self, tenant_id, router_id):

        uri = ROUTER_URI % (tenant_id, router_id)

        self.send_request("DELETE", uri, resource='router',
                          tenant_id=tenant_id, resource_id=router_id)

        LOG.debug(_("Router %(id)s deleted under tenant %(tenant_id)s"),
                  {'id': router_id, 'tenant_id': tenant_id})



# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2013 Big Switch Networks, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
from oslo.config import cfg

import neutron.common.test_lib as test_lib
from neutron.db import api as db
from neutron.plugins.bigswitch import config
from neutron.tests.unit.bigswitch import fake_server

RESTPROXY_PKG_PATH = 'neutron.plugins.bigswitch.plugin'
NOTIFIER = 'neutron.plugins.bigswitch.plugin.AgentNotifierApi'
CALLBACKS = 'neutron.plugins.bigswitch.plugin.RestProxyCallbacks'
CERTFETCH = 'neutron.plugins.bigswitch.servermanager.ServerPool._fetch_cert'
SERVER_MANAGER = 'neutron.plugins.bigswitch.servermanager'
HTTPCON = 'httplib.HTTPConnection'
SPAWN = 'eventlet.GreenPool.spawn_n'
CWATCH = SERVER_MANAGER + '.ServerPool._consistency_watchdog'


class BigSwitchTestBase(object):

    _plugin_name = ('%s.NeutronRestProxyV2' % RESTPROXY_PKG_PATH)

    def setup_config_files(self):
        etc_path = os.path.join(os.path.dirname(__file__), 'etc')
        test_lib.test_config['config_files'] = [os.path.join(etc_path,
                                                'restproxy.ini.test')]
        self.addCleanup(cfg.CONF.reset)
        config.register_config()
        # Only try SSL on SSL tests
        cfg.CONF.set_override('server_ssl', False, 'RESTPROXY')
        cfg.CONF.set_override('ssl_cert_directory',
                              os.path.join(etc_path, 'ssl'), 'RESTPROXY')
        # The mock interferes with HTTP(S) connection caching
        cfg.CONF.set_override('cache_connections', False, 'RESTPROXY')

    def setup_patches(self):
        self.httpPatch = mock.patch(HTTPCON, create=True,
                                    new=fake_server.HTTPConnectionMock)
        self.plugin_notifier_p = mock.patch(NOTIFIER)
        self.callbacks_p = mock.patch(CALLBACKS)
        self.spawn_p = mock.patch(SPAWN)
        self.watch_p = mock.patch(CWATCH)
        self.addCleanup(db.clear_db)
        self.callbacks_p.start()
        self.plugin_notifier_p.start()
        self.httpPatch.start()
        self.spawn_p.start()
        self.watch_p.start()



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""add unique constraint to members

Revision ID: e197124d4b9
Revises: havana
Create Date: 2013-11-17 10:09:37.728903

"""

# revision identifiers, used by Alembic.
revision = 'e197124d4b9'
down_revision = 'havana'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = [
    'neutron.services.loadbalancer.plugin.LoadBalancerPlugin',
    'neutron.plugins.nicira.NeutronServicePlugin.NvpAdvancedPlugin',
]

from alembic import op

from neutron.db import migration


CONSTRAINT_NAME = 'uniq_member0pool_id0address0port'
TABLE_NAME = 'members'


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.create_unique_constraint(
        name=CONSTRAINT_NAME,
        source=TABLE_NAME,
        local_cols=['pool_id', 'address', 'protocol_port']
    )


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.drop_constraint(
        CONSTRAINT_NAME,
        TABLE_NAME,
        type_='unique'
    )



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2012 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Sumit Naiksatam, Cisco Systems, Inc.


from neutron.plugins.common import constants as p_const


FLAT_VLAN_ID = -1
LOCAL_VLAN_ID = -2

# Supported VXLAN features
VXLAN_NONE = 'not_supported'
VXLAN_MCAST = 'multicast_flooding'
VXLAN_UCAST = 'unicast_flooding'


# TODO(rkukura): Eventually remove this function, which provides
# temporary backward compatibility with pre-Havana RPC and DB vlan_id
# encoding.
def interpret_vlan_id(vlan_id):
    """Return (network_type, segmentation_id) tuple for encoded vlan_id."""
    if vlan_id == LOCAL_VLAN_ID:
        return (p_const.TYPE_LOCAL, None)
    elif vlan_id == FLAT_VLAN_ID:
        return (p_const.TYPE_FLAT, None)
    else:
        return (p_const.TYPE_VLAN, vlan_id)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Embrane, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Ivar Lazzaro, Embrane, Inc.


class Operation(object):
    """Defines a series of operations which shall be executed in order.

    the operations expected are procedures, return values are discarded

    """

    def __init__(self, procedure, args=(), kwargs={}, nextop=None):
        self._procedure = procedure
        self.args = args[:]
        self.kwargs = dict(kwargs)
        self.nextop = nextop

    def execute(self):
        args = self.args
        self._procedure(*args, **self.kwargs)
        return self.nextop

    def execute_all(self):
        nextop = self.execute()
        while nextop:
            nextop = self.execute_all()

    def has_next(self):
        return self.nextop is not None

    def add_bottom_operation(self, operation):
        op = self
        while op.has_next():
            op = op.nextop
        op.nextop = operation
